#include <iostream>
#include <ctime>
#include <SDL/SDL.h>
#include <GL/glew.h>
#include <SDL/SDL_framerate.h>
#include "boite.h"
#include "sphere.h"

void Render ();
void Prepare ();
void InitSDL ();
void InitOGL ();
void InitScene ();
void Shutdown ();
void DessinerCube ();
void GetFPS ();

int main (int argc, char *argv [])
{
   InitSDL();
   InitOGL();

   SDL_Event event;
   bool bQuit = false;

   FPSmanager manager;
   SDL_initFramerate (&manager);
   SDL_setFramerate (&manager, 80); // On limite le fps  60 img/s

   while (!bQuit)
   {
      GetFPS ();

      Prepare ();
      Render ();

      SDL_framerateDelay (&manager);

      if (SDL_PollEvent (&event))
      {
         switch (event.type)
         {
            case SDL_QUIT:
               Shutdown ();
               bQuit = true;
               break;
            default:
               break;
         }
      }
    }

   return EXIT_SUCCESS;
}

void GetFPS ()
{
   static char titleBar[25] = {0};
   static int FPS = 0;
   float nextSecond = 0.0f;
   static float prevSecond = 0.0f;

   FPS++;

   nextSecond = SDL_GetTicks() * 0.001f;

   if (nextSecond - prevSecond > 1.0f)
   {
         prevSecond = nextSecond;

         sprintf(titleBar, "%d FPS", FPS);

         SDL_WM_SetCaption (titleBar, NULL);

         FPS = 0;
   }
}

void InitSDL ()
{
   // Initialisation de SDL
   SDL_Init (SDL_INIT_VIDEO);

   SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1); // On active le double buffering

   SDL_Surface * pEcran = SDL_SetVideoMode (640, 480, 32, SDL_OPENGL);

   if (pEcran == NULL)
   {
      std::cerr << "Erreur au paramtrage vido : " << SDL_GetError ();
   }

   atexit (SDL_Quit);
}


// PARTIE OPENGL

const GLint NOMBRE_BOITES = 30;
const GLint NOMBRE_SPHERES = 30;

// Variables globales pour Newton
NewtonWorld * nWorld = NULL;
Sphere * spheres [NOMBRE_SPHERES];
Boite * sol = NULL; // Pointeur vers un objet sol reprsentant,... le sol !
Boite * boites [NOMBRE_BOITES]; // Pointeur vers un objet boite1 reprsentant,... une bote !

void InitNewton ();

void InitOGL ()
{
   glClearColor(0.0, 0.0, 0.3, 0.0);
   glClearDepth(1.0);

   GLenum err = glewInit();

   if (GLEW_OK != err)
      std::cerr << "Erreur dans GLEW.";

   glViewport (0, 0, 640, 480); // On rgle le Viewport

   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   gluPerspective (60.0, 640.0 / 480.0, 0.1, 1000.0);

   glMatrixMode (GL_MODELVIEW);
   glLoadIdentity (); // On rtablit la matrice identit

   glEnable (GL_DEPTH_TEST);
   glEnable (GL_COLOR_MATERIAL);
   glEnable (GL_LIGHTING);

   glEnable (GL_LIGHT0);
   GLfloat LampeDiffuse [] = {1.0f, 1.0f, 1.0f};
   GLfloat LampeAmbient [] = {0.75f, 0.75f, 0.75f};
   GLfloat LampePosition [] = {6.0f, 10.0f, 5.0f, 1.0f};

   glLightfv (GL_LIGHT0, GL_DIFFUSE, LampeDiffuse);
   glLightfv (GL_LIGHT0, GL_SPECULAR, LampeDiffuse);
   glLightfv (GL_LIGHT0, GL_AMBIENT, LampeAmbient);
   glLightfv (GL_LIGHT0, GL_POSITION, LampePosition);

   // On initialise Newton
   InitNewton ();
}

void Prepare ()
{
   NewtonUpdate (nWorld, 1.0f/80.0f);
}

void Render ()
{
   glClearColor(0.0f, 0.0, 0.3, 0.0);
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity ();

   gluLookAt (0.0f, 0.0f, 20.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);

   // Puis quelques rotations afin de bien voir le sol
   glRotatef(15, 1, 0, 0);
   glRotatef(45, 0, 1, 0);

   // On va d'abord dessiner le sol
   sol->Dessiner ();

   // Puis la bote
   for (int i = 0 ; i < NOMBRE_BOITES ; i++)
   {
      boites [i]->Dessiner ();
   }

   for (int i = 0 ; i < NOMBRE_SPHERES ; i++)
   {
      spheres [i]->Dessiner ();
   }

   SDL_GL_SwapBuffers();
}

void Shutdown ()
{
   delete sol, sol = NULL;

   for (int i = 0 ; i < NOMBRE_BOITES ; i++)
      delete boites [i], boites [i] = NULL;

   for (int i = 0 ; i < NOMBRE_SPHERES ; i++)
      delete spheres [i], spheres [i] = NULL;

   NewtonDestroy (nWorld);
}

void InitNewton ()
{
   nWorld = NewtonCreate (NULL, NULL); // On initialise nWorld

   // On dfinit le sol
   CVector taille (10.0f, 0.5f, 10.0f);
   CVector position (-5.0f, 0.0f, 0.0f);
   CVector couleur (100, 100, 100);

   // On alloue la mmoire. Le troisime argument est false pour signifier qu'il sera
   // immobile et donc ne bougera pas
   sol = new Boite ();
   sol->SetColor (couleur);
   sol->Initialiser (nWorld, taille, position, false, 0.0f);

   srand (time (NULL));

   // A prsent le cube
   for (int i = 0 ; i < NOMBRE_SPHERES ; i++)
   {
      GLfloat x, y, z;
      x = -10 + rand()%15;
      y = 1 + rand()%10;
      z = -10 + rand()%15;

      position.ReglerCoordonnees (x, y, z);

      // On slectionne les trois couleurs
      couleur.x = rand() % 256;
      couleur.y = rand() % 256;
      couleur.z = rand() % 256;

      spheres [i] = new Sphere ();
      spheres [i]->Initialiser (nWorld, 1.0f, position, true, 15.0f);
      spheres [i]->SetColor (couleur);
   }

   // A prsent le cube
   for (int i = 0 ; i < NOMBRE_BOITES ; i++)
   {
      GLfloat x, y, z;
      x = -10 + rand()%15;
      y = 1 + rand()%10;
      z = -10 + rand()%15;

      taille.ReglerCoordonnees (1.0f, 1.0f, 1.0f);
      position.ReglerCoordonnees (x, y, z);

      // On slectionne les trois couleurs
      couleur.x = rand() % 256;
      couleur.y = rand() % 256;
      couleur.z = rand() % 256;

      boites [i] = new Boite ();
      boites [i]->Initialiser (nWorld, taille, position, true, 10.0f);
      boites [i]->SetColor (couleur);
   }
}
